import redis
import matplotlib.pylab as plt
import numpy as np
import time

r = redis.StrictRedis(host = 'localhost', port = 6379, db = 0) 

#-------------------------------------------------------------------------------
def quit(): # Завершение работы системы 
  global r
  r.set(b'status', 'q')  
#-------------------------------------------------------------------------------
def vget(key): 
  """Получение значения физической величины. ассоциированной с устройством
     key - ключ устройства в БД (строка байтов)"""
  global r
  ikey = b'i' + key # Ключ хэша с калибровочными данными и др. информацией
  fc = r.hget(ikey, 'fc') # Получим код семейства
  if fc is None:
    print('Неверный ключ')
    return
  name = r.hget(b'codes', fc).decode() # Название устройства
  unit = r.lindex('units', r.hget(b'i' + key, 'ui')).decode() # Ед. измерения
  val = r.lindex(key, -1).decode()
  print('%s, ключ %s, текущее значение %s %s'%(name, key, val, unit))
#-------------------------------------------------------------------------------
def vset(key, val):
  """Установка значения физической величины, ассоциированной с устройством
     key - ключ устройства (строка байтов)
     val - устанавливаемое значение"""
  global r
  ikey = b'i' + key # Ключ хэша с калибровочными данными и др. информацией
  fc = r.hget(ikey, 'fc') # Получим код семейства
  if fc is None:
    print('Неверный ключ')
    return
  fc = int(fc)
  name = r.hget(b'codes', fc).decode() # Название устройства
  unit = r.lindex('units', r.hget(b'i' + key, 'ui')).decode() # Ед. измерения
  if not fc in [0xA2, 0xA3, 0xA4]: # Это не исполнительное устройство
    print('%s с ключем %s не является исполнительным устройством'%(name, key.decode()))
    return
  if fc in [0xA3, 0xA4]: # Это PDM и PWM контроллеры
    val = float(val)
    maxval = float(r.hget(ikey, 'max')) # Макс. значение для данного устройства
    if val > maxval:
      val = maxval
      print('Будет установлено максимальное значение: %.0f %s'%(val, unit))
  elif fc == 0xA2: # Твердотельные реле
    val = int(val)

  r.lset(key, -1, val) # Заменяем текущее значение на новое
  r.rpush(b'must', key) # Добавляем устройство в очередь на обновление
#-------------------------------------------------------------------------------
def cget(key): 
  """Получение калибровочный данных устройством
     key - ключ устройства (строка байтов)"""
  global r
  ikey = b'i' + key # Ключ хэша с калибровочными данными и др. информацией
  fc = r.hget(ikey, 'fc') # Получим код семейства
  if fc is None:
    print('Неверный ключ')
    return
  name = r.hget(b'codes', fc).decode() # Название устройства
  coef = float(r.hget(ikey, 'coef')) # Коэффициент
  offset = int(r.hget(ikey, 'offset')) # Смещение
  ui = int(r.hget(ikey, 'ui')) # Индекс единицы измерения
  unit = r.lindex('units', ui).decode() # Ед. измерения
  print('%s, ключ %s\nкалибровка: coef = %f, offset = %d, ui = %d (%s)'%(name, 
        key, coef, offset, ui, unit))
#-------------------------------------------------------------------------------
def cset(key, coef = 1.0, offset = 0, ui = 0): 
  """Запись калибровочной информации в EEPROM устройства
     key - ключ устройства (строка байтов)
     coef - коэффициент (вещественное число)
     offset - смещение (целое число)
     ui - индекс единицы измерения"""
  global r
  ikey = b'i' + key # Ключ хэша с калибровочными данными и др. информацией
  fc = r.hget(ikey, 'fc') # Получим код семейства
  if fc is None:
    print('Неверный ключ')
    return
  
  r.hset(ikey, b'coef', float(coef)) # Заносим данные в хэш
  r.hset(ikey, b'offset', int(offset))
  r.hset(ikey, b'ui', int(ui))
  r.rpush(b'mucal', key) # Добавляем устройство в очередь для обновления EEPROM
#-------------------------------------------------------------------------------
def plot(th = 3600): 
  """Построение графиков всех температур за истекший период времени
     th - длительность истекшего периода в секундах"""
  global r
  colors = ('b', 'r', 'g', 'c', 'm', 'y', 'k') # Кортеж цветов matplotlib
  ys = [] # Список списков отсчетов температур
  
  ikeys = r.keys('ix*')
  x28lst = [] # Формируем список ключей датчиков температуры DS18B20
  for ikey in ikeys: # Пробегаем по списку всех устройств
    if int(r.hget(ikey, 'fc')) == 0x28:
      x28lst.append(ikey[1:]) # Убираем буковку 'i' в начале
   
  while not r.get(b'lock') is None: time.sleep(0.1) # Ждем конца блокировки
  r.set(b'lock', 1); r.expire(b'lock', 10) # Блокируем БД не более, чем 10 сек

  l = r.llen('time') # Длина истории
  if th > l:
    th = l
  t = r.lrange('time', -th, -1) # Временные отсчеты за последние th секунд
  for key in sorted(x28lst): # Пробегаем по хэшам устройства
    y = r.lrange(key, -th, -1)
    ys.append(y) # Добавим список отсчетов температур
  r.delete(b'lock') # Разблокируем БД

  ci = 0 # Индекс цвета в кортеже цветов
  # Преобразуем списки строк редиски в массивы numpy и добавим их в pylab.plot
  ta = np.array(t).astype(np.float) # Массив отсчетов времени
  t0 = ta[-1]; ta -= t0 # Сдвинем начало отсчета оси времени
  for y in ys:
    ya = np.array(y).astype(np.float) # Массив отсчетов температур
    plt.plot(ta, ya, colors[ci]) # Добавляем эту зависимость к pylab.plot
    ci += 1 # Модернизируем индекс цвета для следующего графика
    if ci > 6: ci = 0 # Графиков может быть много...
  plt.grid(True) 
  plt.show()
#-------------------------------------------------------------------------------
